@@ -41,18 +41,21 @@ class GR_CocoaImage : public GR_RasterImage
 {
 public:
 	GR_CocoaImage(const char* pszName);
-	~GR_CocoaImage();
+	virtual ~GR_CocoaImage();
 
-	virtual UT_sint32	getDisplayWidth(void) const;
-	virtual UT_sint32	getDisplayHeight(void) const;
 	virtual bool		convertToBuffer(UT_ByteBuf** ppBB) const;
 	virtual bool		convertFromBuffer(const UT_ByteBuf* pBB, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight);
 
+    virtual GR_Image *  createImageSegment(GR_Graphics * pG, const UT_Rect & rec);
+	virtual bool hasAlpha (void) const;
+	virtual bool isTransparentAt(UT_sint32 x, UT_sint32 y);
+
 //	void			setData(Fatmap * image) { m_image = image; }
 
    	virtual GRType		getType() const { return m_grtype; }
    	virtual bool		render(GR_Graphics *pGR, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight);
 	
+	void setFromImageRep(NSImageRep *imgRep);
 	NSImage * getNSImage ()
 		{ return m_image; };
 
@@ -64,10 +67,7 @@ private:
 
 	GRType m_grtype;
 
-	UT_sint32 m_iDisplayWidth;
-	UT_sint32 m_iDisplayHeight;
-
-	bool _convertPNGFromBuffer(const NSData* data, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight);
+	bool _convertPNGFromBuffer(NSData* data, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight);
 
 //	bool _convertPNGFromBuffer(const UT_ByteBuf* pBB, UT_sint32 iDisplayWidth, UT_sint32 iDisplayHeight);
 };
