@@ -17,6 +17,12 @@
  * 02111-1307, USA.
  */
 
+/*
+ * Port to Maemo Development Platform 
+ * Author: INdT - Renato Araujo <renato.filho@indt.org.br>
+ */
+
+
 // for gtk_label_parse_uline - nothing we can do about this...
 #undef GTK_DISABLE_DEPRECATED
 
@@ -551,6 +557,8 @@ static void help_button_cb (GObject * button, XAP_Dialog * pDlg)
 
 static void sAddHelpButton (GtkDialog * me, XAP_Dialog * pDlg)
 {
+#ifdef HAVE_HILDON
+#else
   // prevent help button from being added twice
   gint has_button = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (me), "has-help-button"));
 
@@ -570,6 +578,7 @@ static void sAddHelpButton (GtkDialog * me, XAP_Dialog * pDlg)
 
     g_object_set_data (G_OBJECT (me), "has-help-button", GINT_TO_POINTER (1));
   }
+#endif
 }
 
 /*!
@@ -584,9 +593,9 @@ void centerDialog(GtkWidget * parent, GtkWidget * child, bool set_transient_for)
 	  gtk_window_set_transient_for(GTK_WINDOW(child),
 				       GTK_WINDOW(parent));
 
-#ifdef HAVE_GNOME
+#if defined(HAVE_GNOME)
 	gnome_window_icon_set_from_default (GTK_WINDOW(child));
-#else
+#elif !defined(HAVE_HILDON)
 	GdkPixbuf * icon = gtk_window_get_icon(GTK_WINDOW(parent));	
 	if ( NULL != icon )
 	{
@@ -601,7 +610,11 @@ void abiSetupModalDialog(GtkDialog * me, XAP_Frame *pFrame, XAP_Dialog * pDlg, g
 	XAP_UnixFrameImpl * pUnixFrameImpl = static_cast<XAP_UnixFrameImpl *>(pFrame->getFrameImpl());
 	
 	// Get the GtkWindow of the parent frame
+#ifdef HAVE_HILDON 
+	GtkWidget * parentWindow = gtk_widget_get_parent(pUnixFrameImpl->getTopLevelWindow());
+#else	
 	GtkWidget * parentWindow = pUnixFrameImpl->getTopLevelWindow();
+#endif	
 	
 	// connect focus to our parent frame
 	connectFocus(GTK_WIDGET(me),pFrame);
@@ -615,7 +628,6 @@ void abiSetupModalDialog(GtkDialog * me, XAP_Frame *pFrame, XAP_Dialog * pDlg, g
 	
 	// set the default response
 	gtk_dialog_set_default_response ( me, dfl_response ) ;
-	
 	sAddHelpButton (me, pDlg);
 
 	// and make it modal
@@ -692,7 +704,6 @@ void abiSetupModelessDialog(GtkDialog * me, XAP_Frame * pFrame, XAP_Dialog * pDl
 	
 	// set the default response
 	gtk_dialog_set_default_response ( me, dfl_response ) ;
-
 	sAddHelpButton (me, pDlg);
 
 	// and mark it as modeless
