@@ -279,7 +279,6 @@ public:
 	UT_uint32       getLID(void) const
 		{ return m_lid;}
 	void            notifyListeners(AV_ChangeMask mask);
-
 #ifdef FMT_TEST
 	//! Pointer to last instatiated FL_DocLayout. Used for debugging.
 	static		FL_DocLayout* m_pDocLayout;
@@ -354,6 +353,9 @@ private:
 	UT_GenericVector<GR_EmbedManager *> m_vecEmbedManager;
 	bool                m_bAutoGrammarCheck;
 	fl_BlockLayout  *   m_PendingBlockForGrammar;
+	UT_sint32           m_iGrammarCount;
+	bool                m_bFinishedInitialCheck;
+	PT_DocPosition      m_iPrevPos;
 };
 
 #endif /* DOCLAYOUT_H */
