@@ -26,8 +26,9 @@
 PX_ChangeRecord_Strux::PX_ChangeRecord_Strux(PXType type,
 											 PT_DocPosition position,
 											 PT_AttrPropIndex indexAP,
+											 UT_uint32 iXID,
 											 PTStruxType struxType)
-	: PX_ChangeRecord(type, position, indexAP)
+	: PX_ChangeRecord(type, position, indexAP, iXID)
 {
 	xxx_UT_DEBUGMSG(("PX_ChangeRecord_Strux pos = %d \n",position));
 	m_struxType = struxType;
@@ -40,7 +41,7 @@ PX_ChangeRecord_Strux::~PX_ChangeRecord_Strux()
 PX_ChangeRecord * PX_ChangeRecord_Strux::reverse(void) const
 {
 	PX_ChangeRecord_Strux * pcr
-		= new PX_ChangeRecord_Strux(getRevType(),m_position,m_indexAP,m_struxType);
+		= new PX_ChangeRecord_Strux(getRevType(),m_position,m_indexAP,getXID(),m_struxType);
 	UT_ASSERT_HARMLESS(pcr);
 	return pcr;
 }
