@@ -310,28 +310,28 @@ void AP_Win32Dialog_Options::_controlEnable( tControl id, bool value )
 
 
 
-DEFINE2_GET_SET_BOOL(PG_SPELL,SpellCheckAsType);
-DEFINE2_GET_SET_BOOL(PG_SPELL,SpellHideErrors);
-DEFINE2_GET_SET_BOOL(PG_SPELL,SpellSuggest);
-DEFINE2_GET_SET_BOOL(PG_SPELL,SpellMainOnly);
-DEFINE2_GET_SET_BOOL(PG_SPELL,SpellUppercase);
-DEFINE2_GET_SET_BOOL(PG_SPELL,SpellNumbers);
-DEFINE2_GET_SET_BOOL(PG_SPELL,GrammarCheck);
+DEFINE2_GET_SET_BOOL(PG_SPELL,SpellCheckAsType)
+DEFINE2_GET_SET_BOOL(PG_SPELL,SpellHideErrors)
+DEFINE2_GET_SET_BOOL(PG_SPELL,SpellSuggest)
+DEFINE2_GET_SET_BOOL(PG_SPELL,SpellMainOnly)
+DEFINE2_GET_SET_BOOL(PG_SPELL,SpellUppercase)
+DEFINE2_GET_SET_BOOL(PG_SPELL,SpellNumbers)
+DEFINE2_GET_SET_BOOL(PG_SPELL,GrammarCheck)
 
-DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewShowRuler);
-DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewShowStatusBar);
-DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewCursorBlink);
-DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewAll);
-DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewHiddenText);
-DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewUnprintable);
-DEFINE2_GET_SET_BOOL(PG_LAYOUT,SmartQuotesEnable);
+DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewShowRuler)
+DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewShowStatusBar)
+DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewCursorBlink)
+DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewAll)
+DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewHiddenText)
+DEFINE2_GET_SET_BOOL(PG_LAYOUT,ViewUnprintable)
+DEFINE2_GET_SET_BOOL(PG_LAYOUT,SmartQuotesEnable)
 
-DEFINE2_GET_SET_BOOL(PG_PREF,PrefsAutoSave);
-DEFINE2_GET_SET_BOOL(PG_PREF,ShowSplash);
-DEFINE2_GET_SET_BOOL(PG_LANG,OtherDirectionRtl);
-DEFINE2_GET_SET_BOOL(PG_LANG,OtherHebrewContextGlyphs);
-DEFINE2_GET_SET_BOOL(PG_LANG,LanguageWithKeyboard);
-DEFINE2_GET_SET_BOOL(PG_LANG,DirMarkerAfterClosingParenthesis);
+DEFINE2_GET_SET_BOOL(PG_PREF,PrefsAutoSave)
+DEFINE2_GET_SET_BOOL(PG_PREF,ShowSplash)
+DEFINE2_GET_SET_BOOL(PG_LANG,OtherDirectionRtl)
+DEFINE2_GET_SET_BOOL(PG_LANG,OtherHebrewContextGlyphs)
+DEFINE2_GET_SET_BOOL(PG_LANG,LanguageWithKeyboard)
+DEFINE2_GET_SET_BOOL(PG_LANG,DirMarkerAfterClosingParenthesis)
 
 #undef DEFINE_GET_SET_BOOL
 
