@@ -457,9 +457,9 @@ ap_bs_Char CharTable[] =
 	{0x39, /* 9      */ { "insertData",			"",					DO_NOT_USE,		""					}},
 	{0x3a, /* :      */ { "insertData",			"",					"",				""					}},
 	{0x3b, /* ;      */ { "insertData",			"",					"",				""					}},
-	{0x3c, /* <      */ { "insertData",			"fontSizeIncrease",	"",				"insertRLM"			}},
+	{0x3c, /* <      */ { "insertData",			"fontSizeDecrease",	"",				"insertRLM"			}},
 	{0x3d, /* =      */ { "insertData",			"toggleSuper",		"",				""					}},
-	{0x3e, /* >      */ { "insertData",			"fontSizeDecrease",	"",				"insertLRM"			}},
+	{0x3e, /* >      */ { "insertData",			"fontSizeIncrease",	"",				"insertLRM"			}},
 	{0x3f, /* ?      */ { "insertData",			"",					"",				""					}},
 	{0x40, /* @      */ { "insertData",			"",					"",				""					}},
 	{0x41, /* A      */ { "insertData",			"selectAll",		DO_NOT_USE,		""					}},
