@@ -76,7 +76,7 @@
 #include <goffice/graph/gog-object-xml.h>
 #include <goffice/data/go-data-simple.h>
 #include <goffice/graph/gog-renderer-gnome-print.h>
-#include <goffice/utils/format.h>
+#include <goffice/utils/go-format.h>
 #include <gsf/gsf-impl-utils.h>
 
 #include "ut_sleep.h"
