@@ -76,8 +76,12 @@ UT_Confidence_t IE_Imp_Object_Sniffer::supportsMIME (const char * szMIME)
 UT_Confidence_t IE_Imp_Object_Sniffer::recognizeContents(const char * szBuf,
 													   UT_uint32 iNumbytes)
 {
-	char * magic = "<?xml version=\"1.0\"?>\n<GogObject type=\"GogGraph\">";
-	if(strncmp(szBuf,magic,strlen(magic) == 0))
+	char * magic1 = "<?xml version=\"1.0\"?>";
+	char * magic2 = "<GogObject type=\"GogGraph\">";
+	
+	xxx_UT_DEBUGMSG(("magic1 = %s \n",magic1));
+	xxx_UT_DEBUGMSG(("magic2 = %s \n",magic2));
+	if((strstr(szBuf,magic1) != NULL) && (strstr(szBuf,magic2) != NULL))
 	   return UT_CONFIDENCE_PERFECT;
 	return UT_CONFIDENCE_ZILCH;
  
