@@ -129,3 +129,16 @@ void OD_ListenerStateAction::bringUpMostRecentlyPostponedElement(
     m_pState = NULL;
     m_stateName = pStateName;
 }
+
+
+/**
+ * Tells the StreamListener to repeat the current element parsing calls
+ * (startElement, endElement and charData) after having called then all once.
+ * 
+ * Useful for ListenerStates that have to pass twice through a XML element.
+ */
+void OD_ListenerStateAction::repeatElement() {
+    m_action=this->ACTION_REPEAT;
+    m_pState = NULL;
+    m_stateName.clear();
+}
