@@ -35,7 +35,7 @@
 
 // External includes
 #include <locale.h>
-
+#include <ctype.h>
 
 
 /**
@@ -83,6 +83,22 @@ void OD_Style_Style::startElement(const XML_Char* pName,
         
         _parse_style_graphicProperties(ppAtts);
         
+    } else if (!UT_strcmp("style:table-properties", pName)) {
+        
+        _parse_style_tableProperties(ppAtts);
+        
+    } else if (!UT_strcmp("style:table-column-properties", pName)) {
+        
+        _parse_style_tableColumnProperties(ppAtts);
+        
+    } else if (!UT_strcmp("style:table-row-properties", pName)) {
+        
+        _parse_style_tableRowProperties(ppAtts);
+        
+    } else if (!UT_strcmp("style:table-cell-properties", pName)) {
+        
+        _parse_style_tableCellProperties(ppAtts);
+        
     } else if (!UT_strcmp("style:default-style", pName)) {
         
         const XML_Char* pAttr;
@@ -95,6 +111,19 @@ void OD_Style_Style::startElement(const XML_Char* pName,
         m_displayName = m_name = "Normal";
         m_parentStyleName = "None";
         
+    } else if (!UT_strcmp("style:columns", pName)) {
+        
+        const XML_Char* pVal;
+        
+        pVal = UT_getAttribute("fo:column-count", ppAtts);
+        if (pVal) {
+            m_columns = pVal;
+        }
+        
+        pVal = UT_getAttribute("fo:column-gap", ppAtts);
+        if (pVal) {
+            m_columnGap = pVal;
+        }
     }
     
 }
@@ -129,19 +158,23 @@ void OD_Style_Style::_parse_style_style(const XML_Char** ppAtts) {
     
     const XML_Char* pAttr;
     
-    pAttr = UT_getAttribute("style:name", ppAtts);
-    UT_ASSERT(pAttr);
-    m_name = pAttr;
+    if (m_name.empty()) {
+        pAttr = UT_getAttribute("style:name", ppAtts);
+        UT_ASSERT(pAttr);
+        m_name = pAttr;
+    }
 
     pAttr = UT_getAttribute("style:family", ppAtts);
     UT_ASSERT(pAttr);
     m_family = pAttr;
 
-    pAttr = UT_getAttribute("style:display-name", ppAtts);
-    if (pAttr) {
-        m_displayName = pAttr;
-    } else {
-        m_displayName = m_name;
+    if (m_displayName.empty()) {
+        pAttr = UT_getAttribute("style:display-name", ppAtts);
+        if (pAttr) {
+            m_displayName = pAttr;
+        } else {
+            m_displayName = m_name;
+        }
     }
     
     pAttr = UT_getAttribute("style:parent-style-name", ppAtts);
@@ -281,6 +314,7 @@ void OD_Style_Style::_parse_style_paragraphProperties(const XML_Char** ppProps)
 void OD_Style_Style::_parse_style_textProperties(const XML_Char** ppProps) {
     
     const XML_Char* pVal;
+    const XML_Char* pVal2;
     
     pVal = UT_getAttribute("fo:color", ppProps);
     if (pVal) {
@@ -334,13 +368,16 @@ void OD_Style_Style::_parse_style_textProperties(const XML_Char** ppProps) {
         m_fontSize.assign(pVal);
     }
     
-    
-    if (UT_getAttribute("fo:language", ppProps) &&
-        UT_getAttribute("fo:country", ppProps)) {
+    pVal = UT_getAttribute("fo:language", ppProps);
+    pVal2 = UT_getAttribute("fo:country", ppProps);
+    if ( pVal && pVal2 ) {
             
-        m_lang = UT_UTF8String_sprintf ("%s-%s",
-                                    UT_getAttribute("fo:language", ppProps),
-                                    UT_getAttribute("fo:country", ppProps));
+        if (!UT_strcmp(pVal, "none") && !UT_strcmp(pVal2, "none")) {
+            // AbiWord uses "-none-" instead of "none-none";
+            m_lang = "-none-";
+        } else {
+            m_lang = UT_UTF8String_sprintf ("%s-%s", pVal, pVal2);
+        }
     }
     
     
@@ -396,6 +433,100 @@ void OD_Style_Style::_parse_style_graphicProperties(const XML_Char** ppProps) {
 }
 
 
+/**
+ * <style:table-properties />
+ */
+void OD_Style_Style::_parse_style_tableProperties(const XML_Char** ppProps) {
+    const XML_Char* pVal;
+    
+    pVal = UT_getAttribute("fo:background-color", ppProps);
+    if (pVal) {
+        m_backgroundColor = pVal;
+    }
+}
+
+
+/**
+ * <style:table-column-properties />
+ */
+void OD_Style_Style::_parse_style_tableColumnProperties(const XML_Char** ppProps) {
+    const XML_Char* pVal;
+    
+    pVal = UT_getAttribute("style:column-width", ppProps);
+    if (pVal) {
+        m_columnWidth = pVal;
+    }
+}
+
+
+/**
+ * <style:table-row-properties />
+ */
+void OD_Style_Style::_parse_style_tableRowProperties(const XML_Char** ppProps) {
+    const XML_Char* pVal;
+    
+    pVal = UT_getAttribute("style:min-row-height", ppProps);
+    if (pVal) {
+        m_minRowHeight = pVal;
+    }
+    
+    pVal = UT_getAttribute("style:row-height", ppProps);
+    if (pVal) {
+        m_rowHeight = pVal;
+    }
+}
+
+
+/**
+ * <style:table-cell-properties />
+ */
+void OD_Style_Style::_parse_style_tableCellProperties(const XML_Char** ppProps) {
+    const XML_Char* pVal;
+    
+    // If "fo:border" is defined, its value will fill all "fo:border-*"
+    pVal = UT_getAttribute("fo:border", ppProps);
+    if (pVal) {
+        _stripColorLength(m_borderTop_color, m_borderTop_thickness, pVal);
+        
+        m_borderBottom_color = m_borderTop_color;
+        m_borderBottom_thickness = m_borderTop_thickness;
+        
+        m_borderLeft_color = m_borderTop_color;
+        m_borderLeft_thickness = m_borderTop_thickness;
+        
+        m_borderRight_color = m_borderTop_color;
+        m_borderRight_thickness = m_borderTop_thickness;
+        
+    } else {
+        pVal = UT_getAttribute("fo:border-top", ppProps);
+        if (pVal) {
+            _stripColorLength(m_borderTop_color, m_borderTop_thickness, pVal);
+        }
+        
+        pVal = UT_getAttribute("fo:border-bottom", ppProps);
+        if (pVal) {
+            _stripColorLength(m_borderBottom_color, m_borderBottom_thickness, pVal);
+        }
+        
+        pVal = UT_getAttribute("fo:border-left", ppProps);
+        if (pVal) {
+            _stripColorLength(m_borderLeft_color, m_borderLeft_thickness, pVal);
+        }
+        
+        pVal = UT_getAttribute("fo:border-right", ppProps);
+        if (pVal) {
+            _stripColorLength(m_borderRight_color, m_borderRight_thickness, pVal);
+        }
+    }
+    
+    
+    pVal = UT_getAttribute("fo:background-color", ppProps);
+    if (pVal) {
+        m_backgroundColor = pVal;
+    }
+}
+
+
 /**
  * Defines an AbiWord style that is equivalent to this
  * OpenDocument style.
@@ -547,6 +678,7 @@ void OD_Style_Style::buildAbiPropsAttrString() {
     
     // <style:section-properties />
     APPEND_STYLE("columns: ", m_columns);
+    APPEND_STYLE("column-gap: ", m_columnGap);
     
 #undef APPEND_STYLE
 
@@ -590,3 +722,123 @@ const UT_UTF8String* OD_Style_Style::getWrap(bool local) const {
         }
     }
 }
+
+
+/**
+ * If pString is "0.0556in solid #0000ff", rColor will receive "#0000ff" and
+ * rLength "0.0556in".
+ */
+void OD_Style_Style::_stripColorLength(UT_UTF8String& rColor,
+                                       UT_UTF8String& rLength,
+                                       const XML_Char* pString) const {
+                                        
+    UT_uint16 i, start;
+    bool hasWord;
+    
+    rColor.clear();
+    rLength.clear();
+    
+    i = 0;
+    start = 0;
+    hasWord = true;
+    while (pString[i] != 0) {
+        
+        if (hasWord) {
+            if (isspace(pString[i])) {
+                if (_isValidDimensionString(&(pString[start]), i-start)) {
+                    rLength.assign(&(pString[start]), i-start);
+                } else if (pString[start] == '#') {
+                    rColor.assign(&(pString[start]), i-start);
+                }
+                hasWord = false;
+            }
+        } else {
+            if (!isspace(pString[i])) {
+                start = i;
+                hasWord = true;
+            }
+        }
+        
+        i++;
+    };
+    
+    // Process the last word.
+    if (hasWord) {
+        if (_isValidDimensionString(&(pString[start]), i-start)) {
+            rLength.assign(&(pString[start]), i-start);
+        } else if (pString[start] == '#') {
+            rColor.assign(&(pString[start]), i-start);
+        }
+    }
+}
+
+
+/**
+ * This function shouldn't exist. The code should use
+ * UT_isValidDimensionString instead. The problem with the UT function is
+ * that it doesn't check the dimension specifier and only accepts NULL
+ * terminated strings.
+ * 
+ * @param length 0 for NULL terminated strings.
+ */
+bool OD_Style_Style::_isValidDimensionString(const XML_Char* pString,
+                                             UT_uint32 length) const {
+    UT_uint32 i;
+    bool gotDecimalSeparator;
+    
+    gotDecimalSeparator = false;
+    
+    if (length == 0) {
+        length = strlen(pString);
+    }
+    
+    if (length < 3) {
+        // We need at least two characters for the dimension specifier and one
+        // for the number.
+        return false;
+    }
+    
+    for (i=0; i<length; i++) {
+        if ( !isdigit(pString[i]) ) {
+            if (gotDecimalSeparator) {
+                // Already have a decimal separator.
+                // It should be the start of the dimension specifier.
+                break;
+            } else {
+                if (pString[i] == '.' || pString[i] == ',') {
+                    gotDecimalSeparator = true;
+                } else {
+                    // Invalid decimal separator character.
+                    return false;
+                }
+            }
+        }
+    }
+    
+    
+    XML_Char dimStr[100];
+    UT_Dimension dim;
+    UT_uint32 j;
+    
+    if (length - i > 99)  {
+        // A dimension specifier can't be that big.
+        return false;
+    }
+    
+    j = 0;
+    while (i < length) {
+        dimStr[j] = pString[i];
+        
+        i++;
+        j++;
+    }
+    dimStr[j] = 0; // A null terminated string.
+    
+    dim = UT_determineDimension(dimStr, DIM_none);
+    
+    if (dim == DIM_none) {
+        return false;
+    } else {
+        return true;
+    }
+}
