@@ -31,12 +31,15 @@
 #include "OD_Style_MasterPage.h"
 #include "OD_ListenerStateAction.h"
 #include "OD_ListLevelStyle.h"
+#include "OD_NotesConfiguration.h"
 #include "OD_StartTag.h"
 #include "OD_ElementStack.h"
+#include "OD_TableOfContent_ListenerState.h"
 
 // AbiWord includes
 #include <ut_misc.h>
 #include <pd_Document.h>
+#include <pf_Frag_Strux.h>
 
 
 /**
@@ -51,21 +54,34 @@ OD_TextContent_ListenerState::OD_TextContent_ListenerState (
                   m_pStyles(pStyles),
                   m_bAcceptingText(false),
                   m_bInSection(false),
-                  m_row(0),
-                  m_col(0),
-                  m_cel(0),
-                  m_bInTOC(false),
                   m_bOnContentStream(false),
                   m_pCurrentListStyle(NULL),
                   m_listLevel(0),
                   m_alreadyDefinedAbiParagraphForList(false),
-                  m_elementParsingLevel(0)
+                  m_elementParsingLevel(0),
+                  m_pendingNoteAnchorInsertion(false),
+                  m_pCurrentTOCParser(NULL),
+                  m_addedMasterPageSectionProps(false)
 {
     UT_ASSERT(m_pAbiDocument);
     UT_ASSERT(m_pStyles);
 }
 
 
+/**
+ * Destructor
+ */
+OD_TextContent_ListenerState::~OD_TextContent_ListenerState() {
+
+    UT_ASSERT(m_tablesOfContentProps.getItemCount()==0);
+   
+    if (m_tablesOfContentProps.getItemCount() > 0) {
+        UT_VECTOR_PURGEALL(UT_UTF8String*, m_tablesOfContentProps);
+        m_tablesOfContentProps.clear();
+    }
+}
+
+
 /**
  * Called when the XML parser finds a start element tag.
  * 
@@ -76,12 +92,6 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
                                           const XML_Char** ppAtts,
                                           OD_ListenerStateAction& rAction)
 {
-    // if we're inside a TOC, ignore the contents since ODT
-    // includes a copy of the TOC
-    if (m_bInTOC) {
-        return;
-    }
-
     if (!UT_strcmp(pName, "text:section" )) {
 
         const XML_Char* pStyleName = UT_getAttribute ("text:style-name", ppAtts);
@@ -94,7 +104,33 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
 
         _insureInSection(props.utf8_str());
 
-    } else if (!UT_strcmp(pName, "text:p" ) || !UT_strcmp(pName, "text:h" )) {
+    } else if (!UT_strcmp(pName, "text:p" )) {
+        
+        // It's so big that it deserves its own function.
+        _startParagraphElement(pName, ppAtts, rAction);
+        
+    } else if (!UT_strcmp(pName, "text:h" )) {
+
+        const XML_Char* pStyleName;
+        const XML_Char* pOutlineLevel;
+        const OD_Style_Style* pStyle;
+        
+        pStyleName = UT_getAttribute("text:style-name", ppAtts);
+        UT_ASSERT(pStyleName);
+        
+        pOutlineLevel = UT_getAttribute("text:outline-level", ppAtts);
+        if (pOutlineLevel == NULL) {
+            // Headings without a level attribute are assumed to
+            // be at level 1.
+            pOutlineLevel = "1";
+        }
+        
+        pStyle = m_pStyles->getParagraphStyle(pStyleName, m_bOnContentStream);
+        UT_ASSERT(pStyle);
+        
+        m_headingStyles.ins(UT_UTF8String(pOutlineLevel),
+                            pStyle->getDisplayName());
+
         
         // It's so big that it deserves its own function.
         _startParagraphElement(pName, ppAtts, rAction);
@@ -146,12 +182,14 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
         
         _flush ();
         _insureInBlock(NULL);
-
-        m_pAbiDocument->appendStrux(PTX_SectionTOC, NULL);
-        m_pAbiDocument->appendStrux(PTX_EndTOC, NULL);
-
-        m_bInTOC = true;
         
+        UT_ASSERT(m_pCurrentTOCParser == NULL);
+        
+        m_pCurrentTOCParser = new OD_TableOfContent_ListenerState(
+            m_pAbiDocument, m_pStyles, m_rElementStack);
+            
+        rAction.pushState(m_pCurrentTOCParser, false);
+
     } else if (!UT_strcmp(pName, "text:span")) {
         // Write all text that is between the last element tag and this
         // <text:span>
@@ -215,23 +253,6 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
         _insertBookmark (pName, "start");
         _insertBookmark (pName, "end");
         
-    } else if (!UT_strcmp(pName, "table:table")) {
-        
-        _insureInSection(NULL);
-        _openTable (ppAtts);
-        
-    } else if (!UT_strcmp(pName, "table:table-column")) {
-        
-        _openColumn (ppAtts);
-        
-    } else if (!UT_strcmp(pName, "table:table-row")) {
-        
-        _openRow (ppAtts);
-        
-    } else if (!UT_strcmp(pName, "table:table-cell")) {
-        
-        _openCell (ppAtts);
-        
     } else if (!UT_strcmp(pName, "text:date") ||
             !UT_strcmp(pName, "text:time") ||
             !UT_strcmp(pName, "text:page-number") ||
@@ -346,6 +367,101 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
         // We're inside a text-box, parsing its text contents.
         m_bInSection = true;
         m_bOnContentStream = true;
+        
+    } else if (!UT_strcmp(pName, "table:table")) {
+        _insureInSection();
+        rAction.pushState("Table");
+        
+    } else if (!UT_strcmp(pName, "table:table-cell")) {
+        UT_ASSERT(m_elementParsingLevel == 0);
+        
+        // We're inside a table cell, parsing its text contents.
+        m_bInSection = true;
+        m_bOnContentStream = true;
+        
+    }  else if (!UT_strcmp(pName, "text:note")) {
+
+        _flush();
+        m_bAcceptingText = false;
+        
+    } else if (!UT_strcmp(pName, "text:note-body")) {
+
+        const XML_Char* ppAtts[10];
+        bool ok;
+        UT_uint32 id;
+        const OD_NotesConfiguration* pNotesConfig;
+        const OD_Style_Style* pStyle = NULL;
+        const UT_UTF8String* pCitationStyleName;
+        UT_uint8 i;
+        bool isFootnote;
+        const XML_Char* pNoteClass;
+        
+        pNoteClass = m_rElementStack.getStartTag(0)->getAttributeValue("text:note-class");
+        UT_ASSERT(pNoteClass != NULL);
+        
+        if (!UT_strcmp(pNoteClass, "footnote")) {
+            isFootnote = true;
+        } else if (!UT_strcmp(pNoteClass, "endnote")) {
+            isFootnote = false;
+        } else {
+            UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+            // Unrecognized note class.
+        }
+        
+        if (isFootnote) {
+            id = m_pAbiDocument->getUID(UT_UniqueId::Footnote);
+        } else {
+            id = m_pAbiDocument->getUID(UT_UniqueId::Endnote);
+        }
+        UT_UTF8String_sprintf(m_currentNoteId, "%d", id);
+        
+        pNotesConfig = m_pStyles->getNotesConfiguration(pNoteClass);
+        UT_ASSERT(pNotesConfig);
+        
+        pCitationStyleName = pNotesConfig->getCitationStyleName();
+        
+        if (!pCitationStyleName->empty()) {
+            pStyle = m_pStyles->getTextStyle(pCitationStyleName->utf8_str(),
+                        m_bOnContentStream);
+        }
+
+        i = 0;
+        ppAtts[i++] = "type";
+        if (isFootnote) {
+            ppAtts[i++] = "footnote_ref";
+            ppAtts[i++] = "footnote-id";
+        } else {
+            ppAtts[i++] = "endnote_ref";
+            ppAtts[i++] = "endnote-id";
+        }
+        ppAtts[i++] = m_currentNoteId.utf8_str();
+        if (!pCitationStyleName->empty() && pStyle != NULL) {
+            ppAtts[i++] = "style";
+            ppAtts[i++] = pStyle->getDisplayName().utf8_str();
+        }
+        ppAtts[i++] = "props";
+        ppAtts[i++] = "text-position:superscript";
+        ppAtts[i] = 0;
+        
+        ok = m_pAbiDocument->appendObject(PTO_Field, ppAtts);
+        UT_ASSERT(ok);
+        
+        if (isFootnote) {
+            ppAtts[0] = "footnote-id";
+        } else {
+            ppAtts[0] = "endnote-id";
+        }
+        ppAtts[1] = m_currentNoteId.utf8_str();
+        ppAtts[2] = 0;
+        
+        if (isFootnote) {
+            ok = m_pAbiDocument->appendStrux(PTX_SectionFootnote, ppAtts);
+        } else {
+            ok = m_pAbiDocument->appendStrux(PTX_SectionEndnote, ppAtts);
+        }
+        UT_ASSERT(ok);
+        
+        m_pendingNoteAnchorInsertion = true;
     }
     
     m_elementParsingLevel++;
@@ -362,7 +478,13 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
 {
     UT_ASSERT(m_elementParsingLevel >= 0);
     
-    if (!UT_strcmp(pName, "text:section" )) {
+    if (!UT_strcmp(pName, "text:table-of-content")) {
+        
+        m_tablesOfContent.addItem( m_pCurrentTOCParser->getTOCStrux() );
+        m_tablesOfContentProps.addItem( new UT_UTF8String(m_pCurrentTOCParser->getProps()) );
+        DELETEP(m_pCurrentTOCParser);
+        
+    } else if (!UT_strcmp(pName, "text:section" )) {
 
         m_bInSection = false;
 
@@ -371,12 +493,12 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
         _flush ();
         m_bAcceptingText = false;
         
-        // Bring back the possible postponed parsing of a <draw:frame>
-        rAction.bringUpMostRecentlyPostponedElement("Frame", true);
-        
-    } else if (!UT_strcmp(pName, "text:table-of-content")) {
-        
-        m_bInTOC = false;
+        if (!m_rElementStack.hasElement("text:note-body")) {
+            // Footnotes/endnotes can't have frames.
+            
+            // Bring back the possible postponed parsing of a <draw:frame>
+            rAction.bringUpMostRecentlyPostponedElement("Frame", true);
+        }
         
     } else if (!UT_strcmp(pName, "text:span")) {
         
@@ -389,22 +511,6 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
         _flush ();
         m_pAbiDocument->appendObject(PTO_Hyperlink, NULL);
         
-    } else if (!UT_strcmp(pName, "table:table")) {
-        
-        _closeTable ();
-        
-    } else if (!UT_strcmp(pName, "table:table-column")) {
-        
-        _closeColumn ();
-        
-    } else if (!UT_strcmp(pName, "table:table-row")) {
-        
-        _closeRow ();
-        
-    } else if (!UT_strcmp(pName, "table:table-cell")) {
-        
-        _closeCell ();
-        
     } else if (!UT_strcmp(pName, "text:date") ||
         !UT_strcmp(pName, "text:time") ||
         !UT_strcmp(pName, "text:page-number") ||
@@ -429,6 +535,13 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
         
         // We were inside a <office:text> element.
         
+        // That's it, we can't have anymore <text:h> elements.
+        // So, let's define the heading styles on all Abi TOCs (<toc> struxs).
+        _defineAbiTOCHeadingStyles();
+        
+        UT_VECTOR_PURGEALL(UT_UTF8String*, m_tablesOfContentProps);
+        m_tablesOfContentProps.clear();        
+        
         // We can now bring up the postponed parsing (headers/footers and
         // page-anchored frames)
         rAction.bringUpPostponedElements(false);
@@ -455,6 +568,39 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
         
         // We were inside a <draw:text-box> element.
         rAction.popState();
+        
+    } else if (!UT_strcmp(pName, "table:table-cell")) {
+        UT_ASSERT(m_elementParsingLevel == 1);
+        
+        // We were inside a <table:table-cell> element.
+        rAction.popState();
+        
+    } else if (!UT_strcmp(pName, "text:note-body")) {
+        bool ok;
+        const XML_Char* pNoteClass;
+        
+        pNoteClass = m_rElementStack.getStartTag(1)->getAttributeValue("text:note-class");
+        UT_ASSERT(pNoteClass != NULL);
+        
+        if (!UT_strcmp(pNoteClass, "footnote")) {
+            ok = m_pAbiDocument->appendStrux(PTX_EndFootnote, NULL);
+        } else if (!UT_strcmp(pNoteClass, "endnote")) {
+            ok = m_pAbiDocument->appendStrux(PTX_EndEndnote, NULL);
+        } else {
+            UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+            // Unrecognized note class.
+        }
+        
+        UT_ASSERT(ok);
+
+    } else if (!UT_strcmp(pName, "text:note")) {
+        UT_ASSERT(!m_pendingNoteAnchorInsertion);
+        
+        m_pendingNoteAnchorInsertion = false;
+        m_currentNoteId.clear();
+
+        // Back to paragraph text.
+        m_bAcceptingText = true;        
     }
     
     m_elementParsingLevel--;
@@ -467,7 +613,7 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
 void OD_TextContent_ListenerState::charData (
                             const XML_Char* pBuffer, int length)
 {
-    if (pBuffer && length && m_bAcceptingText && !m_bInTOC) {
+    if (pBuffer && length && m_bAcceptingText) {
         m_charData += UT_UCS4String (pBuffer, length, true);
     }
 }
@@ -501,94 +647,6 @@ void OD_TextContent_ListenerState::_flush ()
 }
 
 
-/**
- * 
- */
-void OD_TextContent_ListenerState::_openTable (const XML_Char **ppProps)
-{
-    m_pAbiDocument->appendStrux(PTX_SectionTable, NULL);
-}
-
-
-/**
- * 
- */
-void OD_TextContent_ListenerState::_openColumn (const XML_Char** ppProps)
-{
-    m_col++;
-}
-
-
-/**
- * 
- */
-void OD_TextContent_ListenerState::_openRow (const XML_Char** ppProps)
-{
-    m_row++;
-    m_cel = 0;
-}
-
-
-/**
- * 
- */
-void OD_TextContent_ListenerState::_openCell (const XML_Char** ppProps)
-{
-    UT_String attach;
-
-    attach = UT_String_sprintf(
-        "left-attach: %d; top-attach: %d; right-attach: %d; bot-attach: %d",
-        m_cel, m_row-1, m_cel+1, m_row);
-
-    m_cel++;
-
-    const XML_Char *cell_props[3];
-    cell_props[0] = "props";
-    cell_props[1] = attach.c_str();
-    cell_props[2] = 0;
-    
-    m_pAbiDocument->appendStrux(PTX_SectionCell,cell_props);
-}
-
-
-/**
- * 
- */
-void OD_TextContent_ListenerState::_closeTable ()
-{
-    m_pAbiDocument->appendStrux(PTX_EndTable, NULL);
-
-    m_row = m_cel = m_col = 0;
-}
-
-
-/**
- * 
- */
-void OD_TextContent_ListenerState::_closeColumn ()
-{
-    m_col--;
-}
-
-
-/**
- * 
- */
-void OD_TextContent_ListenerState::_closeRow ()
-{
-    m_col--;
-}
-
-
-/**
- * 
- */
-void OD_TextContent_ListenerState::_closeCell ()
-{
-    m_pAbiDocument->appendStrux(PTX_EndCell,NULL);
-}
-
-
 /**
  * 
  */
@@ -608,9 +666,10 @@ bool OD_TextContent_ListenerState::_pushInlineFmt(const XML_Char ** atts)
             return false;
         }
         
-        if (!m_stackFmtStartIndex.push(start)) {
-            return false;
-        }
+    }
+    
+    if (!m_stackFmtStartIndex.push(start)) {
+        return false;
     }
         
     return true;
@@ -654,14 +713,16 @@ void OD_TextContent_ListenerState::_insureInSection(const XML_Char * pProps)
       return;
     }
     
-    const OD_Style_MasterPage* pMasterPageStyle;
+    const OD_Style_MasterPage* pMasterPageStyle = NULL;
 
-    // For now we just use the Standard page master. AbiWord doesn't support
-    // multiple page formats anyway.
-    pMasterPageStyle = m_pStyles->getMasterPageStyle("Standard");
-    UT_ASSERT(pMasterPageStyle);
-    
-    UT_ASSERT(pMasterPageStyle->getPageLayout());
+    if (!m_addedMasterPageSectionProps) {
+        // For now we just use the Standard page master. AbiWord doesn't support
+        // multiple page formats anyway.
+        pMasterPageStyle = m_pStyles->getMasterPageStyle("Standard");
+        UT_ASSERT(pMasterPageStyle);
+        
+        UT_ASSERT(pMasterPageStyle->getPageLayout());
+    }
     
     // The AbiWord section properties are taken part from the OpenDocument 
     // page layout (from the master page style) and part from the OpenDocument
@@ -670,33 +731,46 @@ void OD_TextContent_ListenerState::_insureInSection(const XML_Char * pProps)
     // TODO: What happens if there are duplicated properties on the page layout
     // and on the section?
 
-    UT_UTF8String allProps(pProps);
-    allProps += pMasterPageStyle->getSectionProps();
+    UT_UTF8String allProps;
     
+    if (!m_addedMasterPageSectionProps) {
+        allProps = pMasterPageStyle->getSectionProps();
+    }
+    
+    if (pProps != NULL) {
+        if (!allProps.empty()) {
+            allProps += "; ";
+        }
+        
+        allProps += pProps;
+    }
 
+    
     const XML_Char* atts[20];
     UT_uint8 i = 0;
     atts[i++] = "props";
     atts[i++] = allProps.utf8_str();
-    
-    if (!pMasterPageStyle->getAWEvenHeaderSectionID().empty()) {
-        atts[i++] = "header-even";
-        atts[i++] = pMasterPageStyle->getAWEvenHeaderSectionID().utf8_str();
-    }
-    
-    if (!pMasterPageStyle->getAWHeaderSectionID().empty()) {
-        atts[i++] = "header";
-        atts[i++] = pMasterPageStyle->getAWHeaderSectionID().utf8_str();
-    }
-    
-    if (!pMasterPageStyle->getAWEvenFooterSectionID().empty()) {
-        atts[i++] = "footer-even";
-        atts[i++] = pMasterPageStyle->getAWEvenFooterSectionID().utf8_str();
-    }
-    
-    if (!pMasterPageStyle->getAWFooterSectionID().empty()) {
-        atts[i++] = "footer";
-        atts[i++] = pMasterPageStyle->getAWFooterSectionID().utf8_str();
+ 
+    if (!m_addedMasterPageSectionProps) {
+        if (!pMasterPageStyle->getAWEvenHeaderSectionID().empty()) {
+            atts[i++] = "header-even";
+            atts[i++] = pMasterPageStyle->getAWEvenHeaderSectionID().utf8_str();
+        }
+        
+        if (!pMasterPageStyle->getAWHeaderSectionID().empty()) {
+            atts[i++] = "header";
+            atts[i++] = pMasterPageStyle->getAWHeaderSectionID().utf8_str();
+        }
+        
+        if (!pMasterPageStyle->getAWEvenFooterSectionID().empty()) {
+            atts[i++] = "footer-even";
+            atts[i++] = pMasterPageStyle->getAWEvenFooterSectionID().utf8_str();
+        }
+        
+        if (!pMasterPageStyle->getAWFooterSectionID().empty()) {
+            atts[i++] = "footer";
+            atts[i++] = pMasterPageStyle->getAWFooterSectionID().utf8_str();
+        }
     }
     
     atts[i] = 0; 
@@ -705,6 +779,10 @@ void OD_TextContent_ListenerState::_insureInSection(const XML_Char * pProps)
 
     m_bInSection = true;
     m_bAcceptingText = false;
+    
+    if (!m_addedMasterPageSectionProps) {
+        m_addedMasterPageSectionProps = true;
+    }
 }
 
 
@@ -732,16 +810,21 @@ void OD_TextContent_ListenerState::_startParagraphElement (const XML_Char* pName
                                           const XML_Char** ppParagraphAtts,
                                           OD_ListenerStateAction& rAction) {
         bool bIsListParagraph = false;
+        const XML_Char* pStyleName;
+        const XML_Char *ppAtts[50];
+        UT_uint8 i;
+        XML_Char listLevel[10];
+        bool ok;
+        UT_UTF8String props;
+        const OD_Style_Style* pStyle;
+        
         
         if (!UT_strcmp(m_rElementStack.getStartTag(0)->getName(), "text:list-item")) {
             // That's a list paragraph.
             bIsListParagraph = true;
         }
 
-        const XML_Char* pStyleName;
-
         pStyleName = UT_getAttribute ("text:style-name", ppParagraphAtts);
-        const OD_Style_Style* pStyle;
 
         if (pStyleName) {
             pStyle = m_pStyles->getParagraphStyle(pStyleName, m_bOnContentStream);
@@ -762,14 +845,8 @@ void OD_TextContent_ListenerState::_startParagraphElement (const XML_Char* pName
         }
 
 
+        i = 0;
 
-        const XML_Char *ppAtts[50];
-        UT_uint8 i = 0;
-        XML_Char listLevel[10];
-        bool ok;
-        UT_UTF8String props;
-        
-        
         if (bIsListParagraph && !m_alreadyDefinedAbiParagraphForList) {
             OD_ListLevelStyle* pListLevelStyle;
             
@@ -866,9 +943,77 @@ void OD_TextContent_ListenerState::_startParagraphElement (const XML_Char* pName
         
             ppAtts[i] = 0; // Marks the end of the attributes list.
             m_pAbiDocument->appendStrux(PTX_Block, (const XML_Char**)ppAtts);
+        }
+
+
+        if (m_pendingNoteAnchorInsertion) {
+            UT_ASSERT(!m_currentNoteId.empty());
+            
+            const XML_Char* pNoteClass;
         
+            pNoteClass = m_rElementStack.getStartTag(1)->
+                                getAttributeValue("text:note-class");
+            UT_ASSERT(pNoteClass != NULL);
+            
+            ppAtts[0] = "type";
+            if (!UT_strcmp(pNoteClass, "footnote")) {
+                ppAtts[1] = "footnote_anchor";
+                ppAtts[2] = "footnote-id";
+            } else if (!UT_strcmp(pNoteClass, "endnote")) {
+                ppAtts[1] = "endnote_anchor";
+                ppAtts[2] = "endnote-id";
+            } else {
+                UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
+                // Unrecognized note class.
+            }
+            ppAtts[3] = m_currentNoteId.utf8_str();
+            ppAtts[4] = 0;
+            
+            ok = m_pAbiDocument->appendObject(PTO_Field, ppAtts);
+            UT_ASSERT(ok);
+            
+            m_pendingNoteAnchorInsertion = false;
         }
 
         // We now accept text
         m_bAcceptingText = true;
 }
+
+
+/**
+ * Defines all toc-source-style* propeties on Abi TOCs (<toc> struxs)
+ */
+void OD_TextContent_ListenerState::_defineAbiTOCHeadingStyles() {
+    UT_uint32 i, j, count;
+    pf_Frag_Strux* pTOCStrux;
+    UT_UTF8String str;
+    UT_UTF8String props;
+    const UT_UTF8String* pStyleName;
+    bool ok;
+    
+    count = m_tablesOfContent.getItemCount();
+    for (i=0; i<count; i++) {
+        pTOCStrux = m_tablesOfContent[i];
+        props = *(m_tablesOfContentProps[i]);
+        
+        for (j=1; j<5; j++) {
+            UT_UTF8String_sprintf(str, "%d", j);
+            pStyleName = m_headingStyles[str];
+
+            if (pStyleName) {
+                UT_UTF8String_sprintf(str, "toc-source-style%d:%s", j,
+                                      pStyleName->utf8_str());
+                
+                if (!props.empty()) {
+                    props += "; ";
+                }
+                props += str;
+            }
+        }
+        
+        ok = m_pAbiDocument->changeStruxAttsNoUpdate(
+                            (PL_StruxDocHandle) pTOCStrux, "props",
+                            props.utf8_str());
+        UT_ASSERT(ok);
+    }
+}
