@@ -1155,7 +1155,7 @@ private:
   UT_UTF8String m_curStyleName;
 
   UT_GenericVector<XML_Char*>	m_vecInlineFmt;
-  UT_Stack		m_stackFmtStartIndex;
+  UT_NumberStack		m_stackFmtStartIndex;
   const OpenWriter_StylesStream_Listener * m_pSSListener;
 
   UT_uint32 m_imgCnt;
@@ -1684,15 +1684,15 @@ private:
 	if (m_vecInlineFmt.addItem(p)!=0)
 	  return false;
       }
-    if (!m_stackFmtStartIndex.push((void*)start))
+    if (!m_stackFmtStartIndex.push(start))
       return false;
     return true;
   }
   
   void _popInlineFmt(void)
   {
-    UT_uint32 start;
-    if (!m_stackFmtStartIndex.pop((void **)&start))
+    UT_sint32 start;
+    if (!m_stackFmtStartIndex.pop(&start))
       return;
     UT_uint32 k;
     UT_uint32 end = m_vecInlineFmt.getItemCount();
