@@ -956,7 +956,7 @@ void s_DocBook_Listener::_openTable(PT_AttrPropIndex api)
 
   m_pie->write("\n<table frame=\"all\">\n");
 
-  UT_String tgroup(UT_String_sprintf("<colgroup cols='%d' align='left' colsep='1' rowsep='1'>\n", nCols));
+  UT_String tgroup(UT_String_sprintf("<tgroup cols='%d' align='left' colsep='1' rowsep='1'>\n", nCols));
   m_pie->write(tgroup.c_str(), tgroup.size());
 
   for (int i = 0; i < nCols; i++)
