@@ -624,8 +624,6 @@ bool s_HRText_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 	case PTX_EndTable:
 	case PTX_SectionCell:
 	case PTX_EndCell:
-	  return true;
-
 	case PTX_EndFrame:
 	case PTX_EndMarginnote:
 	case PTX_EndFootnote:
@@ -634,8 +632,8 @@ bool s_HRText_Listener::populateStrux(PL_StruxDocHandle /*sdh*/,
 	case PTX_SectionFootnote:
 	case PTX_EndEndnote:
 	default:
-		UT_ASSERT_NOT_REACHED();
-		return false;
+		UT_ASSERT(UT_TODO);
+		return true;
 	}
 }
 
