@@ -185,6 +185,8 @@ private:
 
     int							        m_iCurrentListLevel;
     bool								m_bInCell;
+	//needed for correct conversion of complicated tables
+    int							        m_iNumCoveredTableCellsInRow;
 	
 	
 	// HACK HACK HACK
